# Reversegam: ein Klon von Othello/Reversi
import random
import sys
WIDTH = 8  # Das Spiellbrett ist 8 Felder breit.
HEIGHT = 8 # Das Spiellbrett ist 8 Felder hoch.
def drawBoard(board):
    # Gibt das an diese Funktion übergebene Brett aus. Gibt None zurück.
    print(' 12345678')
    print(' +--------+')
    for y in range(HEIGHT):
        print('%s|' % (y+1), end='')
        for x in range(WIDTH):
            print(board[x][y], end='')
        print('|%s' % (y+1))
    print(' +--------+')
    print(' 12345678')

def getNewBoard():
    # Erstelle eine neue, leere Datenstruktur für das Brett.
    board = []
    for i in range(WIDTH):
        board.append([' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '])
    return board
        24.
def isValidMove(board, tile, xstart, ystart):
    # Gibt False zurück, wenn der Spielerzug auf space xstart, ystart ungültig ist.
    # Gibt bei einem gültigen Zug eine Liste der Felder zurück, die dem Spieler zugeschlagen werden, wenn er den Zug macht.
    if board[xstart][ystart] != ' ' or not isOnBoard(xstart, ystart):
        return False
    
    if tile == 'X':
        otherTile = 'O'
    else:
        otherTile = 'X'

    tilesToFlip = []
    for xdirection, ydirection in [[0, 1], [1, 1], [1, 0], [1, -1], [0, -1], [-1, -1], [-1, 0], [-1, 1]]:
        x, y = xstart, ystart
        x += xdirection # Erster Schritt in x-Richtung
        y += ydirection # Erster Schritt in y-Richtung
        while isOnBoard(x, y) and board[x][y] == otherTile:
            # Führt mit der Bewegung in x- und y-Richtung fort.
            x += xdirection
            y += ydirection
            if isOnBoard(x, y) and board[x][y] == tile:
                # Es gibt umzudrehende Steine. Geht in die umgekehrte Richtung, bis das ursprüngliche Feld erreicht ist. Dabei werden alle zwischenliegenden Steine notiert.
                while True:
                    x -= xdirection
                    y -= ydirection
                    if x == xstart and y == ystart:
                        break
                    tilesToFlip.append([x, y])

if len(tilesToFlip) == 0: # Wenn keine Steine umgedreht werden, ist der Zug nicht gültig.
    return False
    return tilesToFlip

def isOnBoard(x, y):
    # Gibt True zurück, wenn die Koordinaten auf dem Brett liegen.
    return x >= 0 and x <= WIDTH - 1 and y >= 0 and y <= HEIGHT - 1

def getBoardWithValidMoves(board, tile):
    # Gibt ein neues Brett mit Punkten als Markierung für die möglichen Züge des Spielers zurück.
    boardCopy = getBoardCopy(board)
    
    for x, y in getValidMoves(boardCopy, tile):
        boardCopy[x][y] = '.'
    return boardCopy

def getValidMoves(board, tile):
    # Gibt eine Liste von [x,y]-Listen mit gültigen Zügen für den gegebenen Spieler auf dem gegebenen Brett zurück.
    validMoves = []
    for x in range(WIDTH):
        for y in range(HEIGHT):
            if isValidMove(board, tile, x, y) != False:
                validMoves.append([x, y])
    return validMoves

def getScoreOfBoard(board):
    # Bestimmt den Punktestand durch Zählen der Steine. Gibt ein Dictionary mit den Schlüsseln 'X' und 'O' zurück.
    xscore = 0
    oscore = 0
    for x in range(WIDTH):
        for y in range(HEIGHT):
            if board[x][y] == 'X':
                xscore += 1
            if board[x][y] == 'O':
                oscore += 1
    return {'X':xscore, 'O':oscore}

def enterPlayerTile():
    # Fragt den Spieler nach dem gewünschten Symbol.
    # Gibt eine Liste mit dem Spielersymbol als erstem und dem Computersymbol als zweitem Element zurück.
    tile = ''
    while not (tile == 'X' or tile == 'O'):
        print('Do you want to be X or O?')
        tile = input().upper()
    
    # Das erste Elemente der Liste ist das Symbol des Spielers, das zweite das des Computers.
    if tile == 'X':
        return ['X', 'O']
    else:
        return ['O', 'X']

def whoGoesFirst():
    # Wählt zufällig aus, wer den ersten Zug hat.
    if random.randint(0, 1) == 0:
        return 'computer'
    else:
        return 'player'

def makeMove(board, tile, xstart, ystart):
    # Platziert den Stein auf xstart, ystart auf dem Brett und dreht alle entsprechenden gegnerischen Steine um.
    # Gibt bei einem ungültigen Zug False zurück und bei einem gültigen True.
    tilesToFlip = isValidMove(board, tile, xstart, ystart)
    
    if tilesToFlip == False:
        return False
    
    board[xstart][ystart] = tile
    for x, y in tilesToFlip:
        board[x][y] = tile
    return True

def getBoardCopy(board):
    # Kopiert die Liste des Spielbretts und gibt die Kopie zurück.
    boardCopy = getNewBoard()
    
    for x in range(WIDTH):
        for y in range(HEIGHT):
            boardCopy[x][y] = board[x][y]

return boardCopy

def isOnCorner(x, y):
    # Gibt True zurück, wenn die Position einer der vier Ecken ist.
    return (x == 0 or x == WIDTH - 1) and (y == 0 or y == HEIGHT - 1)

def getPlayerMove(board, playerTile):
    # Lässt den Spieler einen Zug machen.
    # Gibt den Zug als [x, y] zurück (oder den String 'hints' bzw. 'quit').
    DIGITS1TO8 = '1 2 3 4 5 6 7 8'.split()
    while True:
        print('Enter your move, "quit" to end the game, or "hints" totoggle hints.')
        move = input().lower()
        if move == 'quit' or move == 'hints':
            return move
        
        if len(move) == 2 and move[0] in DIGITS1TO8 and move[1] inDIGITS1TO8:
            x = int(move[0]) - 1
            y = int(move[1]) - 1
            if isValidMove(board, playerTile, x, y) == False:
                continue
            else:
                break
        else:
            print('That is not a valid move. Enter the column (1-8) and then the row (1-8).')
            print('For example, 81 will move on the top-right corner.')

return [x, y]

def getComputerMove(board, computerTile):
    # Ermittelt anhand des übergegebenen Bretts und gegebenen
    # Computersymbols den Zug und gibt ihn als [x, y]-Liste zurück.
    possibleMoves = getValidMoves(board, computerTile)
    random.shuffle(possibleMoves) # Ordnet die Züge zufällig an.
    
    # Belegt nach Möglichkeit eine Ecke.
    for x, y in possibleMoves:
        if isOnCorner(x, y):
            return [x, y]

# Sucht den Zug mit der höchstmöglichen Punktzahl.
bestScore = -1
    for x, y in possibleMoves:
        boardCopy = getBoardCopy(board)
        makeMove(boardCopy, computerTile, x, y)
        score = getScoreOfBoard(boardCopy)[computerTile]
        if score > bestScore:
            bestMove = [x, y]
            bestScore = score
return bestMove

def printScore(board, playerTile, computerTile):
    scores = getScoreOfBoard(board)
    print('You: %s points. Computer: %s points.' % (scores[playerTile], scores[computerTile]))

def playGame(playerTile, computerTile):
    showHints = False
    turn = whoGoesFirst()
    print('The ' + turn + ' will go first.')
    
    # Leert das Brett und beginnt mit dem Platzieren der Steine.
    board = getNewBoard()
    board[3][3] = 'X'
    board[3][4] = 'O'
    board[4][3] = 'O'
    board[4][4] = 'X'
    
    while True:
        playerValidMoves = getValidMoves(board, playerTile)
        computerValidMoves = getValidMoves(board, computerTile)
        
        if playerValidMoves == [] and computerValidMoves == []:
            return board # Das Spiel endet, da keiner ziehen kann.
    
        elif turn == 'player': # Der Spieler ist am Zug.
            if playerValidMoves != []:
                if showHints:
                    validMovesBoard = getBoardWithValidMoves(board, playerTile)
                    drawBoard(validMovesBoard)
                else:
                    drawBoard(board)
                printScore(board, playerTile, computerTile)
                
                move = getPlayerMove(board, playerTile)
                if move == 'quit':
                    print('Thanks for playing!')
                    sys.exit() # Beendet das Programm.
        elif move == 'hints':
            showHints = not showHints
                continue
                else:
                    makeMove(board, playerTile, move[0], move[1])
        turn = 'computer'

elif turn == 'computer': # Der Computer ist am Zug.
    if computerValidMoves != []:
        drawBoard(board)
            printScore(board, playerTile, computerTile)
                
                input('Press Enter to see the computer\'s move.')
                move = getComputerMove(board, computerTile)
                makeMove(board, computerTile, move[0], move[1])
            turn = 'player'



print('Welcome to Reversegam!')

playerTile, computerTile = enterPlayerTile()

while True:
    finalBoard = playGame(playerTile, computerTile)
    
    # Zeigt den Endpunktstand an.
    drawBoard(finalBoard)
    scores = getScoreOfBoard(finalBoard)
    print('X scored %s points. O scored %s points.' % (scores['X'], scores['O']))
    if scores[playerTile] > scores[computerTile]:
        print('You beat the computer by %s points! Congratulations!' % (scores[playerTile] - scores[computerTile]))
    elif scores[playerTile] < scores[computerTile]:
        print('You lost. The computer beat you by %s points.' % (scores[computerTile] - scores[playerTile]))
    else:
        print('The game was a tie!')
    
    print('Do you want to play again? (yes or no)')
    if not input().lower().startswith('y'):
        break
