# Sonar-Schatzsuche

import random
import sys
import math

def getNewBoard():
    # Erstellt eine neue Datenstruktur für das 60x15-Spielbrett
    board = []
    for x in range(60): # Die Hauptliste besteht aus 60 Listen.
        board.append([])
        for y in range(15): # Jede Liste in der Hauptliste besteht aus 15 Ein-Zeichen-Strings.
            # Verwendet der Übersichtlichkeit halber unterschiedliche Zeichen für das Meer.
            if random.randint(0, 1) == 0:
                board[x].append('~')
            else:
                board[x].append('`')
    return board

def drawBoard(board):
    # Zeichnet die Datenstruktur des Spielbretts.
    tensDigitsLine = '    ' # Anfangsleerzeichen für die Zahlen an der linken Seite des Spielbretts.
    for i in range(1, 6):
        tensDigitsLine += (' ' * 9) + str(i)

    # Gibt die Zahlen am oberen Rand des Spielbretts aus.
    print(tensDigitsLine)
    print('   ' + ('0123456789' * 6))
    print()

# Gibt die 15 Reihen aus.
for row in range(15):
    # Einstelligen Zahlen muss ein Leerzeichen vorangestellt werden
    if row < 10:
        extraSpace = ' '
        else:
            extraSpace = ''

    # Erstellt den String für diese Zeile auf dem Brett.
    boardRow = ''
        for column in range(60):
            boardRow += board[column][row]

print('%s%s %s %s' % (extraSpace, row, boardRow, row))

    # Gibt die Zahlen am unteren Rand des Spielbretts aus.
    print()
    print('   ' + ('0123456789' * 6))
    print(tensDigitsLine)

def getRandomChests(numChests):
    # Erstellt eine Liste als Datenstruktur für die Truhen (die jeweils Listen mit den x- und y-Integerkoordinaten als Elementen sind).
    chests = []
    while len(chests) < numChests:
        newChest = [random.randint(0, 59), random.randint(0, 14)]
        if newChest not in chests: # Stellt sicher, dass die Truhe noch nicht vorhanden ist.
            chests.append(newChest)
    return chests

def isOnBoard(x, y):
    # Gibt True zurück, wenn sich die Koordinaten auf dem Spielbrett befinden, sonst False.
    return x >= 0 and x <= 59 and y >= 0 and y <= 14

def makeMove(board, chests, x, y):
    # Ergänzt die Spielbrett-Datenstruktur um Sonarzeichen. Entfernt gefundene Truhen von der Truhenliste.
    # Gibt bei einem ungültigen Zug False zurück.
    # Gibt sonst den String zurück, der aus dem Zug resultiert.
    smallestDistance = 100 # Alle Truhen sind näher als 100.
    for cx, cy in chests:
        distance = math.sqrt((cx - x) * (cx - x) + (cy - y) * (cy - y))

        if distance < smallestDistance: # Wir sind an der nächstliegenden Schutztruhe interessiert
            smallestDistance = distance

smallestDistance = round(smallestDistance)
    
    if smallestDistance == 0:
        # xy ist direkt auf einer Schatztruhe!
        chests.remove([x, y])
        return 'You have found a sunken treasure chest!'
else:
    if smallestDistance < 10:
        board[x][y] = str(smallestDistance)
            return 'Treasure detected at a distance of %s from the sonardevice.' % (smallestDistance)
        else:
            board[x][y] = 'X'
                return 'Sonar did not detect anything. All treasure chests out of range.'

def enterPlayerMove(previousMoves):
    # Ruft den Zug des Spielers ab. Gibt eine zweielementige Liste aus xy-Integerkoordinaten zurück.
    print('Where do you want to drop the next sonar device? (0-59 0-14) (or type quit)')
    while True:
        move = input()
        if move.lower() == 'quit':
            print('Thanks for playing!')
            sys.exit()

        move = move.split()
        if len(move) == 2 and move[0].isdigit() and move[1].isdigit() and isOnBoard(int(move[0]), int(move[1])):
            if [int(move[0]), int(move[1])] in previousMoves:
                print('You already moved there.')
                continue
            return [int(move[0]), int(move[1])]

        print('Enter a number from 0 to 59, a space, then a number from0 to 14.')

def showInstructions():
    print('''Instructions:
You are the captain of the Simon, a treasure-hunting ship. Your current mission
is to use sonar devices to find three sunken treasure chests at the bottom of
the ocean. But you only have cheap sonar that finds distance, not direction.

Enter the coordinates to drop a sonar device. The ocean map will be marked with
how far away the nearest chest is, or an X if it is beyond the sonar device's
range. For example, the C marks are where chests are. The sonar device shows a
3 because the closest chest is 3 spaces away.

        1         2         3
        012345678901234567890123456789012
        
        0 ~~~~`~```~`~``~~~``~`~~``~~~``~`~ 0
        1 ~`~`~``~~`~```~~~```~~`~`~~~`~~~~ 1
        2 `~`C``3`~~~~`C`~~~~`````~~``~~~`` 2
        3 ````````~~~`````~~~`~`````~`~``~` 3
        4 ~`~~~~`~~`~~`C`~``~~`~~~`~```~``~ 4
        
        012345678901234567890123456789012
                1         2         3
(In the real game, the chests are not visible in the ocean.)

Press enter to continue...''')
    input()

    print('''When you drop a sonar device directly on a chest, you retrieve it and the other
sonar devices update to show how far away the next nearest chest is. The chests
are beyond the range of the sonar device on the left, so it shows an X.

        1         2         3
        012345678901234567890123456789012
        
        0 ~~~~`~```~`~``~~~``~`~~``~~~``~`~ 0
        1 ~`~`~``~~`~```~~~```~~`~`~~~`~~~~ 1
        2 `~`X``7`~~~~`C`~~~~`````~~``~~~`` 2
        3 ````````~~~`````~~~`~`````~`~``~` 3
        4 ~`~~~~`~~`~~`C`~``~~`~~~`~```~``~ 4
        
        012345678901234567890123456789012
                1         2         3
        
The treasure chests don't move around. Sonar devices can detect treasure chests
up to a distance of 9 spaces. Try to collect all 3 chests before running out of
sonar devices. Good luck!

Press enter to continue...''')
    input()



print('S O N A R !')
print()
print('Would you like to view the instructions? (yes/no)')
if input().lower().startswith('y'):
    showInstructions()

while True:
    # Spiel wird eingerichtet.
    sonarDevices = 20
    theBoard = getNewBoard()
    theChests = getRandomChests(3)
    drawBoard(theBoard)
    previousMoves = []

    while sonarDevices > 0:
        # Zeigt Sonar- und Truheninventar an.
        print('You have %s sonar device(s) left. %s treasure chest(s)remaining.' % (sonarDevices, len(theChests)))

        x, y = enterPlayerMove(previousMoves)
        previousMoves.append([x, y]) # We must track all moves so thatsonar devices can be updated.

        moveResult = makeMove(theBoard, theChests, x, y)
        if moveResult == False:
            continue
        else:
            if moveResult == 'You have found a sunken treasure chest!':
                # Aktualisiert alle Sonargeräte auf der Karte.
                for x, y in previousMoves:
                    makeMove(theBoard, theChests, x, y)
            drawBoard(theBoard)
            print(moveResult)

        if len(theChests) == 0:
            print('You have found all the sunken treasure chests! Congratulations and good game!')
                break

            sonarDevices -= 1

if sonarDevices == 0:
    print('We\'ve run out of sonar devices! Now we have to turn the ship around and head')
        print('for home with treasure chests still out there! Game over.')
        print(' The remaining chests were here:')
        for x, y in theChests:
            print(' %s, %s' % (x, y))

print('Do you want to play again? (yes or no)')
    if not input().lower().startswith('y'):
        sys.exit()
